@@ -1,6 +1,6 @@
 # gemato: OpenPGP signature support tests
 # (c) 2017-2023 Michał Górny
-# Licensed under the terms of 2-clause BSD license
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 import base64
 import contextlib
@@ -46,8 +46,8 @@ from tests.keydata import (
     VALID_KEY_NOEMAIL, VALID_KEY_NONUTF, COMBINED_PUBLIC_KEYS,
     VALID_KEY_SUBKEY, PRIVATE_KEY, EXPIRED_PUBLIC_KEY, REVOKED_PUBLIC_KEY,
     OTHER_VALID_PUBLIC_KEY, UNSIGNED_PUBLIC_KEY, FORGED_PUBLIC_KEY,
-    UNSIGNED_SUBKEY, FORGED_SUBKEY, SIG_TIMESTAMP, SUBKEY_FINGERPRINT,
-    SUBKEY_SIG_TIMESTAMP, UNEXPIRE_PUBLIC_KEY, OLD_UNEXPIRE_PUBLIC_KEY,
+    UNSIGNED_SUBKEY, FORGED_SUBKEY, SUBKEY_FINGERPRINT,
+    UNEXPIRE_PUBLIC_KEY, OLD_UNEXPIRE_PUBLIC_KEY,
     FORGED_UNEXPIRE_KEY, TWO_SIGNATURE_PUBLIC_KEYS, SECOND_KEY_FINGERPRINT,
     SECOND_VALID_PUBLIC_KEY, TWO_KEYS_ONE_EXPIRED,
     )
@@ -106,6 +106,24 @@ mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
 -----END PGP SIGNATURE-----
 """
 
+POST_EXPIRATION_SIGNED_MANIFEST = f"""
+-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA256
+
+{COMMON_MANIFEST_TEXT}
+-----BEGIN PGP SIGNATURE-----
+
+iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAmPsj28ACgkQE2iA5yp7
+E4R0xAf8CC6uh8VMmv8xlFePEoBYEuSUtDa2hWHJv1sMn90QnszHGG6oo32g2Lje
+H9NRyjOltAG9t0siF/pf57EiKCs9B+Z9zLGYuWlK4gvkHjMHzsoTipUymm2/saEo
+AuoeZvhqNtfU0hCIJsWENtdyMb/hsJIxIOwBjVS/JT5cZlOGjhlyxVO0CS/7FsCp
+GZCeLYPdYXPw2em2DR3Q3NDuNmUY7W3WhJCL14uC+AkU64SnHc13xQ9/go6TQ2ho
+783Jm2f/4ZREYpKMvCgUJvOADSqnfY89hc6B/9JCXn+Zm8a31zgENlJ8DEhN0JMN
+le/JaXEH/AhO6xCOmk8tNQ3QXcNF5w==
+=UGgA
+-----END PGP SIGNATURE-----
+"""
+
 DASH_ESCAPED_SIGNED_MANIFEST = '''
 -----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA256
@@ -225,17 +243,6 @@ def strip_openpgp(text):
     return '\n'.join(lines[start+1:stop-start+2]) + '\n'
 
 
-MANIFESTS_GOOD_SIG = [
-    'SIGNED_MANIFEST',
-    'DASH_ESCAPED_SIGNED_MANIFEST',
-    'SUBKEY_SIGNED_MANIFEST',
-]
-MANIFESTS_BAD_SIG = [
-    'MODIFIED_SIGNED_MANIFEST',
-    'EXPIRED_SIGNED_MANIFEST'
-]
-
-
 # workaround pyflakes' limitations
 _ = COMBINED_PUBLIC_KEYS
 _ = EXPIRED_PUBLIC_KEY
@@ -256,7 +263,14 @@ _ = VALID_KEY_SUBKEY
 
 
 @pytest.mark.parametrize('manifest_var',
-                         MANIFESTS_GOOD_SIG + MANIFESTS_BAD_SIG)
+                         ["SIGNED_MANIFEST",
+                          "DASH_ESCAPED_SIGNED_MANIFEST",
+                          "SUBKEY_SIGNED_MANIFEST",
+                          "MODIFIED_SIGNED_MANIFEST",
+                          "EXPIRED_SIGNED_MANIFEST",
+                          "TWO_SIGNATURE_MANIFEST",
+                          "POST_EXPIRATION_SIGNED_MANIFEST",
+                          ])
 def test_noverify_goodish_manifest_load(manifest_var):
     """Test Manifest files that should succeed (OpenPGP disabled)"""
     m = ManifestFile()
@@ -390,6 +404,9 @@ MANIFEST_VARIANTS = [
     ('SIGNED_MANIFEST', 'COMBINED_PUBLIC_KEYS', None),
     ('DASH_ESCAPED_SIGNED_MANIFEST', 'VALID_PUBLIC_KEY', None),
     ('SUBKEY_SIGNED_MANIFEST', 'VALID_KEY_SUBKEY', None),
+    ("POST_EXPIRATION_SIGNED_MANIFEST", "VALID_PUBLIC_KEY", None),
+    # == Manifest signed before the key expired ==
+    ("SIGNED_MANIFEST", "EXPIRED_PUBLIC_KEY", None),
     # == Manifest with two signatures ==
     ("TWO_SIGNATURE_MANIFEST", "TWO_SIGNATURE_PUBLIC_KEYS", None),
     ("TWO_SIGNATURE_MANIFEST", "VALID_PUBLIC_KEY", OpenPGPVerificationFailure),
@@ -407,7 +424,7 @@ MANIFEST_VARIANTS = [
     # == bad keys ==
     ('SIGNED_MANIFEST', None,
      OpenPGPVerificationFailure),
-    ('SIGNED_MANIFEST', 'EXPIRED_PUBLIC_KEY',
+    ("POST_EXPIRATION_SIGNED_MANIFEST", "EXPIRED_PUBLIC_KEY",
      OpenPGPExpiredKeyFailure),
     ('SIGNED_MANIFEST', 'REVOKED_PUBLIC_KEY',
      OpenPGPRevokedKeyFailure),
@@ -453,13 +470,19 @@ def assert_signature(sig: OpenPGPSignatureList,
     elif manifest_var == 'SUBKEY_SIGNED_MANIFEST':
         assert len(sig) == 1
         assert sig.fingerprint == SUBKEY_FINGERPRINT
-        assert sig.timestamp == SUBKEY_SIG_TIMESTAMP
+        assert sig.timestamp == datetime.datetime(2020, 8, 25, 12, 40, 12)
+        assert sig.expire_timestamp is None
+        assert sig.primary_key_fingerprint == KEY_FINGERPRINT
+    elif manifest_var == "POST_EXPIRATION_SIGNED_MANIFEST":
+        assert len(sig) == 1
+        assert sig.fingerprint == KEY_FINGERPRINT
+        assert sig.timestamp == datetime.datetime(2023, 2, 15, 7, 53, 19)
         assert sig.expire_timestamp is None
         assert sig.primary_key_fingerprint == KEY_FINGERPRINT
     else:
         assert len(sig) == 1
         assert sig.fingerprint == KEY_FINGERPRINT
-        assert sig.timestamp == SIG_TIMESTAMP
+        assert sig.timestamp == datetime.datetime(2017, 11, 8, 9, 1, 26)
         assert sig.expire_timestamp is None
         assert sig.primary_key_fingerprint == KEY_FINGERPRINT
 
@@ -830,16 +853,16 @@ REFRESH_VARIANTS = [
     ('SIGNED_MANIFEST', 'EXPIRED_PUBLIC_KEY', KEY_FINGERPRINT,
      'UNEXPIRE_PUBLIC_KEY', None),
     # ...but only with a new signature
-    ('SIGNED_MANIFEST', 'EXPIRED_PUBLIC_KEY', KEY_FINGERPRINT,
-     'OLD_UNEXPIRE_PUBLIC_KEY', OpenPGPExpiredKeyFailure),
+    ("POST_EXPIRATION_SIGNED_MANIFEST", "EXPIRED_PUBLIC_KEY", KEY_FINGERPRINT,
+     "OLD_UNEXPIRE_PUBLIC_KEY", OpenPGPExpiredKeyFailure),
     # make sure server can't malicously inject or replace key
     ('SIGNED_MANIFEST', 'OTHER_VALID_PUBLIC_KEY', OTHER_KEY_FINGERPRINT,
      'VALID_PUBLIC_KEY', OpenPGPKeyRefreshError),
     ('SIGNED_MANIFEST', 'OTHER_VALID_PUBLIC_KEY', OTHER_KEY_FINGERPRINT,
      'COMBINED_PUBLIC_KEYS', OpenPGPRuntimeError),
     # test that forged keys are rejected
-    ('SIGNED_MANIFEST', 'EXPIRED_PUBLIC_KEY', KEY_FINGERPRINT,
-     'FORGED_UNEXPIRE_KEY', OpenPGPExpiredKeyFailure),
+    ("POST_EXPIRATION_SIGNED_MANIFEST", "EXPIRED_PUBLIC_KEY", KEY_FINGERPRINT,
+     "FORGED_UNEXPIRE_KEY", OpenPGPExpiredKeyFailure),
     ('SUBKEY_SIGNED_MANIFEST', 'VALID_PUBLIC_KEY', KEY_FINGERPRINT,
      'FORGED_SUBKEY', OpenPGPVerificationFailure),
     ('SUBKEY_SIGNED_MANIFEST', 'VALID_PUBLIC_KEY', KEY_FINGERPRINT,
@@ -1117,9 +1140,10 @@ def test_verify_detached(tmp_path, key_var, two_sigs):
             with open(tmp_path / "sig.bin", "wb") as f:
                 f.write(base64.b64decode(TWO_SIGNATURES))
 
-            sig = openpgp_env.verify_detached(
-                tmp_path / "sig.bin", tmp_path / "data.bin",
-                require_all_good=two_sigs)
+            with open(tmp_path / "data.bin", "rb") as f:
+                sig = openpgp_env.verify_detached(
+                    tmp_path / "sig.bin", f,
+                    require_all_good=two_sigs)
 
             assert_signature(sig, "TWO_SIGNATURE_MANIFEST",
                              expect_both=two_sigs)
